<?php

namespace Controller;

use Adresse;
use Client;
use Form\AddressForm;
use Pays;
use Texte;
use Tools;
use Panier;

use Symfony\Component\Routing\Annotation\Route;

/**
 * @Route("/account/address", defaults = {"role" = "customer"})
 */
class AddressController extends Controller
{

    /**
     * @Route("/", name="address_account", methods = { "GET" })
     *
     * @return mixed
     */
    public function index_account()
    {
        $pays = new Pays();
        $flashBag = $this->getFlashBag();
        $adresse = new Adresse();
        $client = $this->getCurrentClient();

        /* template variables */
        $tplVars = array();
        $tplVars['pays'] = $pays->get();
        $tplVars['pays_id'] = Pays::FRANCE;
        $tplVars['civilite'] = $client->getCivilite();
        $tplVars['client'] = $client->toArray();

        $arrAdresse = $adresse->get(array("client_id" => $client->client_id));
        $tplVars["adresses"] = $arrAdresse;

        //default
        if (!$flashBag->has("post")) {
            $addressId = $this->request->get("adresse_id");
            $found = array();

            foreach ($arrAdresse as $ad) {
                if ($ad["adresse_id"] == $addressId) {
                    $found = $ad;
                    break;
                }
            }

            if (empty($found)) {
                $found = $arrAdresse[0];
            }

            $tplVars["post"] = $found;
        } else {
            $tplVars["post"] = $flashBag->get("post");
        }

        $flashes = $flashBag->all();

        return $this->fetch("address/index_account.tpl", array_merge($flashes, $tplVars));
    }

    /**
     * @Route("/order", name="address_order", methods = { "GET" })
     *
     * @return mixed
     */
    public function index_order()
    {
        $panier = Panier::getInstance();
        $pays = new Pays();
        $flashBag = $this->getFlashBag();
        $adresse = new Adresse();
        $client = $this->getCurrentClient();

        /* template variables */
        $tplVars = array();
        $tplVars['pays'] = $pays->get();
        $tplVars['pays_id'] = Pays::FRANCE;
        $tplVars['civilite'] = $client->getCivilite();
        $tplVars['client'] = $client->toArray();
        $tplVars['same_addr'] = $panier->getSameAddress();

        $arrAdresse = $adresse->get(array("client_id" => $client->client_id));
        $tplVars["adresses"] = $arrAdresse;

        //default
//        if (!$flashBag->has("post")) {
//            $addressId = $this->request->get("adresse_id");
//            $found = array();
//
//            foreach ($arrAdresse as $ad) {
//                if ($ad["adresse_id"] == $addressId) {
//                    $found = $ad;
//                    break;
//                }
//            }
//
//            if (empty($found)) {
//                $found = $arrAdresse[0];
//            }
//
//            $tplVars["post"] = $found;
//        }

        // assign currently used addresses for each type
        $panier = Panier::getInstance();
        $types = array("shipping", "billing");
        foreach ($types as $t) {
            $method = "getAddress" . ucfirst(strtolower($t));
            if (method_exists($panier, $method)) {
                $tplVars["post"][strtolower($t)] = $panier->$method()->toArray();
            }
        }

        $flashes = $flashBag->all();

        return $this->fetch("address/index_order.tpl", array_merge($flashes, $tplVars));
    }

    /**
     * @Route("/create", name="address_create", methods = { "POST" })
     *
     * @return mixed
     */
    public function create()
    {
        $texte = new Texte();
        $adresse = new Adresse();
        $flashBag = $this->getFlashBag();
        $req = $this->request->request;
        $params = $req->all();
        $form = new AddressForm();

        if ($error = $form->handle($this->request)) {
            $flashBag->add("error", $error);
        }

        if (!$flashBag->has("error")) {
            // update
            $addr = new Adresse();
            $addr->hydrate($params);
            $addr->client_id = $this->getCurrentClient()->client_id;
            $params["adresse_id"] = $adresse->save($addr->toArray());

            // set default
            if (!empty($params["adresse_default"]))
                Tools::setDefault('adresse', 'client', $_SESSION['client_id'], $addr->adresse_id);

            $flashBag->add("success", $texte->getOneOrCreate("succAddrCreate", "Votre adresse a été créée."));
        }

        /* template variables */
        $tplVars = array();
        if ($req->has("type")) {
            $tplVars["type"] = $req->get("type");
        }
        $arrAdresse = $adresse->get(array("client_id" => $this->getCurrentClient()->client_id));
        $tplVars["adresses"] = $arrAdresse;

        $flashBag->set("post", $params);

        $flashes = $flashBag->all();

        $this->template->assign(array_merge($tplVars, $flashes));

        /* json */
        $json = array();
        $json["html"] = array();
        $json["html"]["selector"] = $this->template->fetch("address/selector.tpl");

        if ($flashBag->has("error")) {
            $json["error"] = $flashBag->get("error");
        }

        if ($flashBag->has("success")) {
            $json["success"] = $flashBag->get("success");
        }

        if (!empty($params["adresse_id"])) {
            $json["create"] = $params["adresse_id"];
        }

        $json = array_merge($json, $flashes);

        return $this->response($json);
    }

    /**
     * @Route("/{id}/update", name="address_update", methods = { "POST" }, requirements = { "id" = "^[1-9][0-9]*$" })
     *
     * @return mixed
     */
    public function update()
    {
        $id = $this->request->get("id");
        $flashBag = $this->getFlashBag();
        $req = $this->request->request;
        $texte = new Texte();
        $adresse = new Adresse($id);
        $pays = new Pays();
        $customer = new Client();

        $params = $this->request->request->all();
        $form = new AddressForm();

        if ($error = $form->handle($this->request)) {
            $flashBag->add("error", $error);
        }

        if (!$flashBag->has("error")) {
            // update
            $adresse->hydrate($params);
            $adresse->client_id = $this->getCurrentClient()->client_id;
            $adresse->update($adresse->toArray());
            $adresse->load($id);

            // set default
            if (!empty($params["adresse_default"])) {
                Tools::setDefault('adresse', 'client', $this->getCurrentClient()->client_id, $id);
            }

            $flashBag->add("success", $texte->getOneOrCreate("succAddrChang", "Votre adresse a été modifiée."));

            if ($this->request->get('origin') == 'order') {
                $panier = Panier::getInstance();
                if ($this->request->get('type') == 'shipping') {
                    $panier->setAddressShipping($adresse);
                } else {
                    $panier->setAddressBilling($adresse);
                }
            }
        }

        $flashBag->set("post", $params);

        /* template variables */
        $tplVars = array();
        $tplVars['pays'] = $pays->get();
        $tplVars['civilite'] = $customer->getCivilite();
        $tplVars["address"] = $adresse->toArray();
        $tplVars["post"] = array_merge($params, $tplVars["address"]);

        $arrAdresse = $adresse->get(array("client_id" => $this->getCurrentClient()->client_id));
        $tplVars["adresses"] = $arrAdresse;
        if ($req->has("type")) {
            $tplVars["type"] = $req->get("type");
        }
        $flashes = $flashBag->all();
        $this->template->assign(array_merge($flashes, $tplVars));

        /* json */
        $json = array();
        $json["html"] = array();

        if (!$flashBag->has("error")) {
            $json["update"] = $adresse->adresse_id;
        }


        $json["html"]["view"] = $this->template->fetch("address/view.tpl");
        $json["html"]["update"] = $this->template->fetch($req->has("type") ? "address/form_order.tpl" : "address/form_account.tpl");
        $json["html"]["selector"] = $this->template->fetch("address/selector.tpl");
        $json = array_merge($json, $flashes);
        return $this->response($json);
    }

    /**
     * @Route("/{id}/delete", name="address_delete", methods = { "POST" }, requirements = { "id" = "^[1-9][0-9]*$" })
     *
     * @return mixed
     */
    public function delete()
    {
        $id = $this->request->get("id");
        $texte = new Texte();
        $flashBag = $this->getFlashBag();
        $adresse = new Adresse($id);
        $adresse->delete(array(
                "adresse_id" => $adresse->adresse_id,
                "client_id"  => $this->getCurrentClient()->client_id)
        );
        $flashBag->add("success", $texte->getOneOrCreate("succAddrDelete", "Votre adresse a été supprimée."));

        /* template variables */
        $tplVars = array();
        $arrAdresse = $adresse->get(array("client_id" => $this->getCurrentClient()->client_id));
        $tplVars["adresses"] = $arrAdresse;

        $flashes = $flashBag->all();
        $this->template->assign(array_merge($this->getFlashAssign(array("post", "error", "success")), $tplVars));
        
        /* json */
        $json = array();
        $json["html"] = array();
        $json["delete"] = true;
        $json["html"]["selector"] = $this->template->fetch("address/selector.tpl");
        $json = array_merge($json, $flashes);
        return $this->response($json);
    }

    /**
     * @Route("/select", name="address_select", methods = { "POST" })
     *
     * @return mixed
     */
    public function select()
    {
        $req = $this->request->request;
        if ($req->has("addr_shipping")) {
            $id = $req->get("addr_shipping");
        } elseif ($req->get("addr_billing")) {
            $id = $req->get("addr_billing");
        } else {
            $id = $req->get("adresse_id");
        }

        if (empty($id)) {
            return $this->response();
        }

        $pays = new Pays();
        $address = new Adresse($id);
        $customer = $this->getCurrentClient();
        $flashBag = $this->getFlashBag();

        /* template variables */
        $tplVars = array();
        $tplVars['pays'] = $pays->get();
        $tplVars['civilite'] = $customer->getCivilite();
        $tplVars["post"] = array_merge($address->toArray(), $req->all());
        $tplVars["address"] = $address->toArray();
        $arrAdresse = $address->get(array("client_id" => $customer->client_id));
        $tplVars["adresses"] = $arrAdresse;

        // type uniquement dans le process commande
        if ($req->has("type")) {
            $tplVars["type"] = $req->get("type");

            $panier = Panier::getInstance();
            $setBilling = false;
            if ($req->get("type") == 'shipping') {
                $panier->setAddressShipping($address);

                // same addresses for shipping and billing
                if($req->has("same_address") && $req->get("same_address") == 1) {
                    $setBilling = $panier->getSameAddress();
                    $tplVars["same_address"] = $req->get("same_address");
                }
            }

            if ($req->get("type") == 'billing' || $setBilling) {
                $panier->setAddressBilling(clone $address);
            }

//            $tplVars["post"]["adresse_id_shipping"] = $panier->getAddressBilling()->adresse_id;
//            $tplVars["post"]["adresse_id_billing"] = $panier->getAddressShipping()->adresse_id;

            // assign currently used addresses for each type
            $types = array("shipping", "billing");
            foreach ($types as $t) {
                $method = "getAddress" . ucfirst(strtolower($t));
                if (method_exists($panier, $method)) {
                    $tplVars["post"][strtolower($t)] = $panier->$method()->toArray();
                }
            }
        }
        // todo : check si adresse existe et assigner error
        $flashes = $flashBag->all();
        $this->template->assign(array_merge($flashes, $tplVars));

        /* json */
        $json = array();
        $json["html"] = array();
        $json["html"]["view"] = $this->template->fetch("address/view.tpl");
        $json["html"]["update"] = $this->template->fetch($req->has("type") ? "address/form_order.tpl" : "address/form_account.tpl");
        $json["html"]["selector"] = $this->template->fetch("address/selector.tpl");

        return $this->response($json);
    }
}