<?php

namespace Shopbydedi\Shipping;

use SoapClient;

class SoColissimo implements RelayInterface
{

    protected $login;
    protected $mdp;
    protected $pudoFOId;
    protected $cleSHA;
    protected $trFirstOrder;
    protected $wsdl;

    public function __construct($config)
    {
        $this->login = $config['socolissimo']['login'];
        $this->mdp = $config['socolissimo']['mdp'];
        $this->pudoFOId = $config['socolissimo']['pudofoid'];
        $this->cleSHA = $config['socolissimo']['clesha'];

        $this->trFirstOrder = 0;

        $this->wsdl = 'https://ws.colissimo.fr/pointretrait-ws-cxf/PointRetraitServiceWS/2.0?wsdl';
    }

    /**
     * @param array $formvars
     * @return Relay[]
     */
    public function getList($formvars = array())
    {
        $soap = new SoapClient($this->wsdl);

        if(empty($formvars['ville'])) $formvars['ville'] = '';
        if(empty($formvars['code_postal'])) $formvars['code_postal'] = '';
        if(empty($formvars['poids'])) $formvars['poids'] = '';
        if(empty($formvars['pays'])) $formvars['pays'] = 'FR';
        if(empty($formvars['date'])) $formvars['date'] = date("d")."/".date("m")."/".date('Y');

        $param = array(
            'accountNumber' => $this->login,
            'password' => $this->mdp,
            'zipCode' => $formvars['code_postal'],
            'city' => $formvars['ville'],
            'weight' => $formvars['poids'],
            'countryCode' => $formvars['pays'],
            'shippingDate' => $formvars['date']
        );

        $result = $soap->findRDVPointRetraitAcheminement($param);

        /*var_dump($result);
        exit;*/

        $relais = array();
        if (empty($result->return->errorCode)) {
            // Requete ok, on a des pointe relais
            foreach ($result->return->listePointRetraitAcheminement as $key => $value) {
                $relais[] = $this->formatRelay((array)$value);
            }
        }

        return $relais;
    }

    /**
     * @param string $relayId
     * @param string $countryIso
     * @return Relay
     */
    public function getRelay($relayId, $countryIso = '')
    {
        $soap = new SoapClient($this->wsdl);

        $param = array(
            'accountNumber' => $this->login,
            'password' => $this->mdp,
            'id' => $relayId,
            'date' => date("d")."/".date("m")."/".date('Y')
        );

        $result = $soap->findPointRetraitAcheminementByID($param);

        $relay = new Relay();
        if (empty($result->return->errorCode)) {
            $relay = $this->formatRelay((array) $result->return->pointRetraitAcheminement);
        }

        return $relay;
    }

    /**
     * @param array $data
     * @return Relay
     */
    public function formatRelay(array $data)
    {
        $relay = new Relay();
        $relay->id = trim($data['identifiant']);
        $relay->name = trim($data['nom']);
        $relay->addr1 = trim($data['adresse1']);
        $relay->addr2 = trim($data['adresse2']);
        $relay->zipcode = trim($data['codePostal']);
        $relay->city = trim($data['localite']);
        $relay->country = trim($data['libellePays']);
        $relay->latitude = $data['coordGeolocalisationLatitude'];
        $relay->longitude = $data['coordGeolocalisationLongitude'];
        $relay->distance = $data['distanceEnMetre'];

        /*$horaires = array('Lundi' => array(), 'Mardi' => array(), 'Mercredi' => array(), 'Jeudi' => array(), 'Vendredi' => array(), 'Samedi' => array(), 'Dimanche' => array());
        foreach ($horaires as $key => $hor) {
            foreach ($data['Horaires_' . $key]->string as $k => $horMr) {
                $res = $this->parseHoraire($horMr);
                if (!empty($res)) {
                    $horaires[$key][$k] = $res;
                }
            }
        }

        $relay->horaires = $horaires;*/

        return $relay;
    }
}